"""
just checks that the worker methods call the right functions
and some data ends up in the right place ... otherwise not very detailed
"""
import json
import logging

from lxml import etree
import pytest
from inventory_provider.tasks import worker


class MockedRedis(object):

    db = {}

    def __init__(self, *args, **kwargs):
        pass

    def set(self, name, value):
        assert isinstance(value, str)
        if name.startswith('netconf:'):
            etree.fromstring(value)
        elif name.startswith('snmp-interfaces:'):
            obj = json.loads(value)
            assert isinstance(obj, list)
        MockedRedis.db[name] = value


@pytest.fixture
def mocked_worker_module(mocker, data_config):

    worker.InventoryTask.config = data_config
    worker.InventoryTask.logger = logging.getLogger()

    MockedRedis.db = {}

    mocker.patch(
        'inventory_provider.tasks.common.redis.StrictRedis',
        MockedRedis)


def test_netconf_refresh_config(
        mocked_worker_module, mocker, router, cached_test_data):

    def _mocked_load_juniper_netconf_config(hostname, _):
        return etree.XML(cached_test_data['netconf:' + hostname])

    mocker.patch(
        'inventory_provider.tasks.worker.juniper.load_config',
        _mocked_load_juniper_netconf_config)

    assert 'netconf:' + router not in MockedRedis.db
    worker.netconf_refresh_config(router)
    assert MockedRedis.db['netconf:' + router]


def test_snmp_refresh_interfaces(
        mocked_worker_module, mocker, router, cached_test_data):

    def _mocked_snmp_interfaces(hostname, community, _):
        return json.loads(cached_test_data['snmp-interfaces:' + hostname])

    mocker.patch(
        'inventory_provider.tasks.worker.snmp.get_router_interfaces',
        _mocked_snmp_interfaces)

    assert 'snmp-interfaces:' + router not in MockedRedis.db
    worker.snmp_refresh_interfaces(router, 'fake-community')
    assert MockedRedis.db['snmp-interfaces:' + router]
